#include "em_device.h"
#include "em_cmu.h"
#include "em_emu.h"
#include "em_gpio.h"
#include "em_letimer.h"

#define LETIMER0_FREQ CMU_ClockFreqGet(cmuClock_LETIMER0)
#define OUT_FREQ 1										// desired PWM frequency (in Hz)

void EMU_setup(void)
{
	EMU_DCDCInit_TypeDef dcdcInit = EMU_DCDCINIT_DEFAULT;
	EMU_DCDCInit(&dcdcInit);							// init DC/DC converter
}

void CMU_setup(void)
{
	CMU_LFXOInit_TypeDef lfxoInit = CMU_LFXOINIT_DEFAULT;
	CMU_LFXOInit(&lfxoInit);
	SystemLFXOClockSet(32768);
	CMU_ClockSelectSet(cmuClock_HF, cmuSelect_HFRCO);	// select 19 MHz HFRCO as clock source
	CMU_ClockEnable(cmuClock_HFLE, true);				// enable LE clock access
	CMU_ClockSelectSet(cmuClock_LFA, cmuSelect_LFXO);
	CMU_ClockSelectSet(cmuClock_LFE, cmuSelect_LFXO);
}

void GPIO_setup(void)									// configure GPIO ports
{
	CMU_ClockEnable(cmuClock_GPIO, true);
	GPIO_PinModeSet(gpioPortD, 9, gpioModePushPull, 0);	// red LED control
}

void LETIMER0_setup(void)
{
	LETIMER_Init_TypeDef letimerInit = LETIMER_INIT_DEFAULT;
	CMU_ClockEnable(cmuClock_LETIMER0, true);

	// Reload COMP0 on underflow, pulse output, and run in repeat mode
	letimerInit.comp0Top = true;
	letimerInit.ufoa0 = letimerUFOAToggle;
	letimerInit.repMode = letimerRepeatFree;
	letimerInit.topValue = CMU_ClockFreqGet(cmuClock_LETIMER0) / OUT_FREQ;

	// Need REP0 != 0 to run PWM
	LETIMER_RepeatSet(LETIMER0, 0, 1);

	// Set COMP0 to control duty cycle
	LETIMER_CompareSet(LETIMER0, 0,
	       CMU_ClockFreqGet(cmuClock_LETIMER0) / OUT_FREQ);

	// Enable LETIMER0 output0 on PD9 (Loc 17)
	LETIMER0->ROUTEPEN |=  LETIMER_ROUTEPEN_OUT0PEN;
	LETIMER0->ROUTELOC0 |= LETIMER_ROUTELOC0_OUT0LOC_LOC17;

	// Initialize and enable LETIMER
	LETIMER_Init(LETIMER0, &letimerInit );
}
